;;; compatibility COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

command_compatibility:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Wait for compatibility command
    RCALL   wait_lpt_byte
; Check which compatibility command is in buffer
compat_dmx_data_block:
    CPI     rdata,'D'
    BRNE    compat_low_dmx_data
    RJMP    compat_command_dmx_data_block
compat_low_dmx_data:
    CPI     rdata,'d'
    BRNE    compat_high_dmx_data
    RJMP    compat_command_low_dmx_data
compat_high_dmx_data:
    CPI     rdata,'e'
    BRNE    compat_flush
    RJMP    compat_command_high_dmx_data
compat_flush:
    CPI     rdata,'F'
    BRNE    compat_init
    RJMP    compat_command_flush
compat_init:
    CPI     rdata,'I'
    BRNE    compat_start_code
    RJMP    compat_command_init
compat_start_code:
    CPI     rdata,'S'
    BRNE    compat_break_time
    RJMP    compat_command_start_code
compat_break_time:
    CPI     rdata,'T'
    BRNE    compat_channel_count
    RJMP    compat_command_break_time
compat_channel_count:
    CPI     rdata,'C'
    BRNE    compat_clear_output
    RJMP    compat_command_channel_count
compat_clear_output:
    CPI     rdata,'Z'
    BRNE    compat_sync
    RJMP    compat_command_clear_output
compat_sync:
    CPI     rdata,0x1B
    BRNE    compat_no_match
; Wait for new data, as this is a sync character
    RJMP    start
; Command unrecognized
compat_no_match:
    set_error_led
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
    RJMP    start

;;; dmx_data_block COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

compat_command_dmx_data_block:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop

; Retrieve high byte of number of first channel
    RCALL   wait_lpt_byte
    MOV     temp1,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop

; Retrieve low byte of number of first channel
    RCALL   wait_lpt_byte
    MOV     temp2,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop

; Retrieve number of channels
    RCALL   wait_lpt_byte
    MOV     temp3,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop

; Fill up temporary pointer with address of output table (plus 1 as DMX output table contains start code)
    LDI     ptr_temp_h,high(output_min + 1)
    LDI     ptr_temp_l,low(output_min + 1)
; Correct temporary pointer with specified number of the first channel (0 = first channel)
    ADD     ptr_temp_l,temp2
    ADC     ptr_temp_h,temp1

; Initialize channel counter
    LDI     temp2,0

dmx_data_block_loop:
; Check output table limit, as only 255 output channels are supported
    CPI     ptr_temp_h,high(output_max+1)
    LDI     temp1,low(output_max+1)
    CPC     ptr_temp_l,temp1
    BRLO    dmx_data_block_save
dmx_data_block_ignore:
; Passed output table limit; ignore provided values
; Retrieve channel's new value (but ignore it)
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
    RJMP    dmx_data_block_continue
dmx_data_block_save:
; Retrieve channel's new value
    RCALL   wait_lpt_byte
; Store channel value into DMX output table
    ST      Z+,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
dmx_data_block_continue:
; Increase channel counter
    INC     temp2
; Check whether channel counter reached specified number of channels
    CP      temp2,temp3
    BRNE    dmx_data_block_loop
; Specified number of channels reached; wait for new data
    RJMP    start

;;; low_dmx_data COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

compat_command_low_dmx_data:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve channel number
    RCALL   wait_lpt_byte
    MOV     temp1,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Check if the specified channel is channel number 255
    CPI     temp1,255
    BRNE    low_dmx_data_save
low_dmx_data_ignore:
; Ignore provided value, as only 255 output channels are supported by this firmware (excluding start code)
; Retrieve channel's new value (but ignore it)
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Wait for new data
    RJMP    start
low_dmx_data_save:
; Fill up temporary pointer with address of output table (plus 1 as DMX output table contains start code)
    LDI     ptr_temp_h,high(output_min + 1)
    LDI     ptr_temp_l,low(output_min + 1)
; Correct temporary pointer with specified channel number (0 = first channel)
    ADD     ptr_temp_l,temp1
    ADC     ptr_temp_h,dummy
; Retrieve channel's new value
    RCALL   wait_lpt_byte
; Store channel value into DMX output table
    ST      Z,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; high_dmx_data COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This simply is a dummy command which ignores the provided data,
; as only 255 output channels are supported by this firmware.

compat_command_high_dmx_data:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve channel number
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve channel's new value
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; flush COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

compat_command_flush:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop

; Initialize EEPROM address for break time byte
    LDI     temp1,high(rom_break_time)
    LDI     temp2,low(rom_break_time)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read break time from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new break time
    IN      break_time,EEDR

; Initialize EEPROM address for output start code byte
    LDI     temp1,high(rom_channels)
    LDI     temp2,low(rom_channels)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read start code output byte from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new start code value
    IN      temp1,EEDR
; Fill up temporary pointer with address of output table;
; the start code is to be stored in the first byte of the table
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
; Store new start code value in output table
    ST      Z,temp1

; Wait for new data
    RJMP    start

;;; init COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

compat_command_init:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Disable interrupts to prevent interrupts executing during initialization
    CLI
; Place interface back in initial state
    RJMP    reset

;;; start_code COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

compat_command_start_code:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve channel number
    RCALL   wait_lpt_byte
; Fill up temporary pointer with address of output table;
; the start code is to be stored in the first byte of the table
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
; Retrieve new start code value
    RCALL   wait_lpt_byte
; Store start code value into DMX output table
    ST      Z,rdata
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; break_time COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; The break_time command cannot be supported in compatibility mode,
; as 1 is specified for 50 microseconds, 2 for 100 microseconds etc.,
; rather than that the number of microseconds is specified directly.
; Therefore instead this command will restore the break time value
; that's stored in the EEPROM.

compat_command_break_time:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve new break time (but ignore it)
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop

; Initialize EEPROM address for break time byte
    LDI     temp1,high(rom_break_time)
    LDI     temp2,low(rom_break_time)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Read break time from EEPROM by enabling EERE bit of EECR
    SBI     EECR,EERE
; Store new break time
    IN      break_time,EEDR

; Wait for new data
    RJMP    start

;;; channel_count COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This command isn't supported in the original firmware for the LPT-DMX
; convertor yet, so no support is needed here either. The two values
; will be received but will simply be ignored, like the original firmware
; does.

compat_command_channel_count:
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve high byte of channel count (but ignore it)
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Retrieve low byte of channel count (but ignore it)
    RCALL   wait_lpt_byte
; Acknowledge LPT data
    RCALL   ack_lpt
    reset_str_flipflop
; Wait for new data
    RJMP    start

;;; clear_output COMPATIBILITY COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

compat_command_clear_output:
; Clear DMX output table
    RCALL   empty_output
; Wait for new data
    RJMP    start
