;;; write_eeprom COMMAND ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Writing to the EEPROM takes a couple of seconds.
; As it should never be interrupted, the error LED will
; be lit during the process.

command_write_eeprom:
    reset_str_flipflop

; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready

; Initialize EEPROM address; address 0 contains no data as it might cause problems according to the errata
    LDI     temp1,high(rom_input_status)
    LDI     temp2,low(rom_input_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Retrieve receiver enabled status ('R' = enabled; 'r' = disabled)
    RCALL   wait_lpt_byte
; Write receiver enabled status
    OUT     EEDR,rdata
    reset_str_flipflop
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_input:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_input

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_output_status)
    LDI     temp2,low(rom_output_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Retrieve transmitter enabled status ('T' = enabled; 't' = disabled)
    RCALL   wait_lpt_byte
; Write transmitter enabled status
    OUT     EEDR,rdata
    reset_str_flipflop
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_output:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_output

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mux_status)
    LDI     temp2,low(rom_mux_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Retrieve mux enabled status ('E' = enabled; 'e' = disabled)
    RCALL   wait_lpt_byte
; Write override enabled status
    OUT     EEDR,rdata
    reset_str_flipflop
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_mux:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_mux

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_merge_status)
    LDI     temp2,low(rom_merge_status)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Retrieve merge status ('X' = enabled; 'x' = disabled)
    RCALL   wait_lpt_byte
; Write merge status
    OUT     EEDR,rdata
    reset_str_flipflop
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_merge:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_merge

; There will be no more parallel input during this EEPROM write, so the error led can be set, indicating an EEPROM write taking place
    set_error_led

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_break_time)
    LDI     temp2,low(rom_break_time)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write break time
    OUT     EEDR,break_time
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_break:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_break

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mab_time)
    LDI     temp2,low(rom_mab_time)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write MAB time
    OUT     EEDR,mab_time
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_mab:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_mab

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_output_count)
    LDI     temp2,low(rom_output_count)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write output count
    OUT     EEDR,output_count
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_outputc:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_outputc

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mux_address)
    LDI     temp2,low(rom_mux_address)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write mux address
    OUT     EEDR,mux_address
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_muxaddr:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_muxaddr

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_input_startcode)
    LDI     temp2,low(rom_input_startcode)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write input startcode to listen for on DMX input
    OUT     EEDR,input_startcode
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_startc:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_startc

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_input_wildcard)
    LDI     temp2,low(rom_input_wildcard)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Check if the input start code wildcard flag is set;
; if so write '*' to EEPROM, otherwise write '#'
    SBRC    flags,input_wildcard
    RJMP    input_wildcard_enabled
; The wildcard is disabled
    LDI     temp1,'#'
    RJMP    write_input_wildcard
input_wildcard_enabled:
; The wildcard is enabled
    LDI     temp1,'*'
write_input_wildcard:
; Write input wildcard status
    OUT     EEDR,temp1
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_wildcrd:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_wildcrd

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mux_invert)
    LDI     temp2,low(rom_mux_invert)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Check if the mux invert flag is set;
; if so write 'N' to EEPROM, otherwise write 'n'
    SBRC    flags,mux_invert
    RJMP    mux_invert_enabled
; The mux invert flag is disabled
    LDI     temp1,'n'
    RJMP    write_mux_invert
mux_invert_enabled:
; The mux invert flag is enabled
    LDI     temp1,'N'
write_mux_invert:
; Write input wildcard status
    OUT     EEDR,temp1
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_mux_inv:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_mux_inv

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_min_h)
    LDI     temp2,low(rom_mass_output_min_h)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write mass output minimum high value
    OUT     EEDR,mass_output_min_h
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_massmnh:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_massmnh

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_min_l)
    LDI     temp2,low(rom_mass_output_min_l)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write mass output minimum low value
    OUT     EEDR,mass_output_min_l
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_massmnl:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_massmnl

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_max_h)
    LDI     temp2,low(rom_mass_output_max_h)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write mass output maximum high value
    OUT     EEDR,mass_output_max_h
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_massmxh:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_massmxh

; Initialize EEPROM address for next byte
    LDI     temp1,high(rom_mass_output_max_l)
    LDI     temp2,low(rom_mass_output_max_l)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Write mass output maximum low value
    OUT     EEDR,mass_output_max_l
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_massmxl:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_massmxl

; Initialize EEPROM address for channel values; bytes 1 up to 99 are reserved for individual bytes
    LDI     temp1,high(rom_channels - 1)
    LDI     temp2,low(rom_channels - 1)
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Initialize temporary pointer to point to first entry of DMX output table
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
write_eeprom_loop:
; Increment EEPROM address
    IN      temp2,EEARL
    IN      temp1,EEARH
    INC     temp2
; Check whether the INC caused an overflow
    BRNE    write_eeprom_no_overflow
; There was an overflow
    INC     temp1
write_eeprom_no_overflow:
    OUT     EEARH,temp1
    OUT     EEARL,temp2
; Put data in EEPROM data register
    LD      temp1,Z+
    OUT     EEDR,temp1
; Write data to EEPROM by enabling EEMWE and EEWE bits of EECR
    CLI
    SBI     EECR,EEMWE
    SBI     EECR,EEWE
    SEI     
; Wait for EEWE bit in EECR to go low, indicating that EEPROM is ready
write_eeprom_wait_ready_chans:
    SBIC    EECR,EEWE
    RJMP    write_eeprom_wait_ready_chans
; Check output table limit
    CPI     ptr_temp_h,high(output_max+1)
    LDI     temp1,low(output_max+1)
    CPC     ptr_temp_l,temp1
    BRNE    write_eeprom_loop

; Passed output table limit; EEPROM writing complete, so reset error led
    reset_error_led
; Wait for new data
    RJMP    start
