;;; MAIN LOOP ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This loop waits for a command character to be received
; from the parallel port and acts accordingly.

start:
    RCALL   wait_lpt_byte_main
; Check which command is in buffer
analyse_command:
    reset_error_led
check_compatibility:
    CPI     rdata,0x1B
    BRNE    check_input
    RJMP    command_compatibility
check_input:
    CPI     rdata,'i'
    BRNE    check_output
    RJMP    command_input
check_output:
    CPI     rdata,'o'
    BRNE    check_bulk_input
    RJMP    command_output
check_bulk_input:
    CPI     rdata,'I'
    BRNE    check_bulk_output
    RJMP    command_bulk_input
check_bulk_output:
    CPI     rdata,'O'
    BRNE    check_mass_output
    RJMP    command_bulk_output
check_mass_output:
    CPI     rdata,'Q'
    BRNE    check_set_mass_output_limits
    RJMP    command_mass_output
check_set_mass_output_limits:
    CPI     rdata,'1'
    BRNE    check_get_input_startcode
    RJMP    command_set_mass_output_limits
check_get_input_startcode:
    CPI     rdata,'K'
    BRNE    check_get_input_count
    RJMP    command_get_input_startcode
check_get_input_count:
    CPI     rdata,'G'
    BRNE    check_get_time_since_input
    RJMP    command_get_input_count
check_get_time_since_input:
    CPI     rdata,'Z'
    BRNE    check_get_mux_value
    RJMP    command_get_time_since_input
check_get_mux_value:
    CPI     rdata,'Y'
    BRNE    check_enable_input
    RJMP    command_get_mux_value
check_enable_input:
    CPI     rdata,'R'
    BRNE    check_disable_input
    RJMP    command_enable_input
check_disable_input:
    CPI     rdata,'r'
    BRNE    check_enable_output
    RJMP    command_disable_input
check_enable_output:
    CPI     rdata,'T'
    BRNE    check_disable_output
    RJMP    command_enable_output
check_disable_output:
    CPI     rdata,'t'
    BRNE    check_enable_mux
    RJMP    command_disable_output
check_enable_mux:
    CPI     rdata,'E'
    BRNE    check_disable_mux
    RJMP    command_enable_mux
check_disable_mux:
    CPI     rdata,'e'
    BRNE    check_force_mux_off
    RJMP    command_disable_mux
check_force_mux_off:
    CPI     rdata,'F'
    BRNE    check_no_force_mux_off
    RJMP    command_force_mux_off
check_no_force_mux_off:
    CPI     rdata,'f'
    BRNE    check_reset
    RJMP    command_no_force_mux_off
check_reset:
    CPI     rdata,'-'
    BRNE    check_defaults
    RJMP    command_reset
check_defaults:
    CPI     rdata,'P'
    BRNE    check_finish
    RJMP    command_defaults
check_finish:
    CPI     rdata,'^'
    BRNE    check_clear_all
    RJMP    command_finish
check_clear_all:
    CPI     rdata,'C'
    BRNE    check_clear_input
    RJMP    command_clear_all
check_clear_input:
    CPI     rdata,'8'
    BRNE    check_clear_output
    RJMP    command_clear_input
check_clear_output:
    CPI     rdata,'9'
    BRNE    check_set_break_time
    RJMP    command_clear_output
check_set_break_time:
    CPI     rdata,'B'
    BRNE    check_set_mab_time
    RJMP    command_set_break_time
check_set_mab_time:
    CPI     rdata,'M'
    BRNE    check_set_output_startcode
    RJMP    command_set_mab_time
check_set_output_startcode:
    CPI     rdata,'D'
    BRNE    check_set_output_count
    RJMP    command_set_output_startcode
check_set_output_count:
    CPI     rdata,'L'
    BRNE    check_enable_input_wildcard
    RJMP    command_set_output_count
check_enable_input_wildcard:
    CPI     rdata,'*'
    BRNE    check_disable_input_wildcard
    RJMP    command_enable_input_wildcard
check_disable_input_wildcard:
    CPI     rdata,'#'
    BRNE    check_set_input_startcode
    RJMP    command_disable_input_wildcard
check_set_input_startcode:
    CPI     rdata,'S'
    BRNE    check_set_mux_address
    RJMP    command_set_input_startcode
check_set_mux_address:
    CPI     rdata,'A'
    BRNE    check_inverted_mux
    RJMP    command_set_mux_address
check_inverted_mux:
    CPI     rdata,'N'
    BRNE    check_non_inverted_mux
    RJMP    command_inverted_mux
check_non_inverted_mux:
    CPI     rdata,'n'
    BRNE    check_enable_merge
    RJMP    command_non_inverted_mux
check_enable_merge:
    CPI     rdata,'X'
    BRNE    check_disable_merge
    RJMP    command_enable_merge
check_disable_merge:
    CPI     rdata,'x'
    BRNE    check_read_eeprom
    RJMP    command_disable_merge
check_read_eeprom:
    CPI     rdata,'V'
    BRNE    check_write_eeprom
    RJMP    command_read_eeprom
check_write_eeprom:
    CPI     rdata,'W'
    BRNE    check_echo
    RJMP    command_write_eeprom
check_echo:
    CPI     rdata,'%'
    BRNE    check_zero
    RJMP    command_echo
check_zero:
; A zero command may be received if the computer is powered off or booted.
; This command will be recognized but will be ignored, so that the error LED will not remain lit.
    CPI     rdata,0x00
    BRNE    check_no_match
    RJMP    start
; Command unrecognized
check_no_match:
    set_error_led
    reset_str_flipflop
    RJMP    start
