.include "routine_read_eeprom.inc"

;;; LPT ACKNOWLEDGE ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; Only used for backward compatibility. Keeps ACK enabled for
; 7.75 microseconds. As there is plenty of free program memory
; anyway, the delay can be implemented without using a loop.

ack_lpt:
    set_ack

; 60 NOP instructions for a delay of 7.5 microseconds
    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    NOP
    NOP
    NOP
    NOP
    NOP

    reset_ack
    RET

;;; empty_tables ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine clears both the DMX input and output tables
; by filling them up with zero bytes.

empty_tables:
; Fill up temporary pointer with address of table
    LDI     ptr_temp_h,high(table_min)
    LDI     ptr_temp_l,low(table_min)
empty_tables_zeroise:
; Store a zero value in the table
    ST      Z+,dummy
    CPI     ptr_temp_h,high(table_max+1)
    LDI     temp1,low(table_max+1)
    CPC     ptr_temp_l,temp1
    BRNE    empty_tables_zeroise
    RET

;;; empty_input ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine clears only the DMX input table
; by filling it up with zero bytes.

empty_input:
; Fill up temporary pointer with address of table
    LDI     ptr_temp_h,high(input_min)
    LDI     ptr_temp_l,low(input_min)
empty_input_zeroise:
; Store a zero value in the table
    ST      Z+,dummy
    CPI     ptr_temp_h,high(input_max+1)
    LDI     temp1,low(input_max+1)
    CPC     ptr_temp_l,temp1
    BRNE    empty_input_zeroise
    RET

;;; empty_output ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine clears only the DMX output table
; by filling it up with zero bytes.

empty_output:
; Fill up temporary pointer with address of table
    LDI     ptr_temp_h,high(output_min)
    LDI     ptr_temp_l,low(output_min)
empty_output_zeroise:
; Store a zero value in the table
    ST      Z+,dummy
    CPI     ptr_temp_h,high(output_max+1)
    LDI     temp1,low(output_max+1)
    CPC     ptr_temp_l,temp1
    BRNE    empty_output_zeroise
    RET

;;; reset_parameters ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;

reset_parameters:
; Default break time is 88 microsec, so the timer value to be stored is 256 - 88 + 2 = 170
    LDI     temp1,256 - default_break_time + 2
    MOV     break_time,temp1
; Initialise MAB value, when set it should be decreased by one
    LDI     temp1,default_mab_time - 1
    MOV     mab_time,temp1
; Initialise output count to 255, which is the maximum
    LDI     temp1,255
    MOV     output_count,temp1
; Initialise multiplexer address to 0
    LDI     temp1,0
    MOV     mux_address,temp1
; Initialise input startcode to 0
    LDI     temp1,0
    MOV     input_startcode,temp1
; Initially unset input wildcard flag
    CLT
    BLD     flags,input_wildcard
; Initially unset mux inverted flag
    CLT
    BLD     flags,mux_invert
; Initialise mass output transfer limit values, the maximum limit should be increased by one
    LDI     temp1,high(output_min + 1) ; skip the start code by default
    MOV     mass_output_min_h,temp1
    LDI     temp1,low(output_min + 1)
    MOV     mass_output_min_l,temp1
    LDI     temp1,high(output_max + 1)
    MOV     mass_output_max_h,temp1
    LDI     temp1,low(output_max + 1)
    MOV     mass_output_max_l,temp1
    RET

;;; wait_lpt_byte ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine is used to retrieve a single character
; from the DATA pins of the parallel port.

; QSTR is high after a strobe occurred, until it is reset using _RESETSTR
; _STR is low while strobing
; STR (PC) is high while strobing

wait_lpt_byte:
; Wait for the STR flipflop to be triggered
    skip_if_str_flipflop
    RJMP    wait_lpt_byte
    set_error_led
wait_lpt_byte2:
; Wait for STR to have gone back low
    skip_if_not_str
    RJMP    wait_lpt_byte2
    reset_error_led
    store_parallel_data
    RET

;;; wait_lpt_byte_main ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine is identical to the wait_lpt_byte routine, however
; this loop updates the status of DMX channels 1 up to 8 if the
; "override" setting is enabled, while the other routine does not.

wait_lpt_byte_main:
; Wait for the STR flipflop to be triggered
    skip_if_not_str_flipflop
    RJMP    wait_lpt_byte_main_trig

; Check if the force mux off flag if set; if it is, do not go on
; checking whether multiplexing should be enabled, but temporarily do
; no multiplexing no matter what's the mux setting
    SBRC    flags,force_mux_off
    RJMP    wait_lpt_byte_main
; Check if mux flag if set; only if it is,
; update channel values from secondary parallel connector
    SBRS    flags,mux_status
    RJMP    wait_lpt_byte_main

; Update status from additional parallel pins
; Store temporary value indicating "on"
    LDI     temp1,255
; Initialize temporary pointer to first value in output table
    LDI     ptr_temp_h,high(output_min + 1)
    LDI     ptr_temp_l,low(output_min + 1)
; Correct temporary pointer with configured multiplexer start address
    ADD     ptr_temp_l,mux_address
    ADC     ptr_temp_h,dummy
; Check if the multiplexer inversion flag is set and use the correct code accordingly
    SBRC    flags,mux_invert
    RJMP    mux_inverted
mux_normal:
.include "handle_parallel_pins_normal.inc"
    RJMP    wait_lpt_byte_main
mux_inverted:
.include "handle_parallel_pins_inverted.inc"
    RJMP    wait_lpt_byte_main

wait_lpt_byte_main_trig:
    set_error_led
wait_lpt_byte_main2:
; Wait for STR to have gone back low
    skip_if_str
    RJMP    wait_lpt_byte_main_trig2

; Check if the force mux off flag if set; if it is, do not go on
; checking whether multiplexing should be enabled, but temporarily do
; no multiplexing no matter what's the mux setting
    SBRC    flags,force_mux_off
    RJMP    wait_lpt_byte_main2
; Check if mux flag if set; only if it is,
; update channel values from secondary parallel connector
    SBRS    flags,mux_status
    RJMP    wait_lpt_byte_main2

; Update status from additional parallel pins
; Store temporary value indicating "on"
    LDI     temp1,255
; Initialize temporary pointer to first value in output table
    LDI     ptr_temp_h,high(output_min + 1)
    LDI     ptr_temp_l,low(output_min + 1)
; Correct temporary pointer with configured multiplexer start address
    ADD     ptr_temp_l,mux_address
    ADC     ptr_temp_h,dummy
; Check if the multiplexer inversion flag is set and use the correct code accordingly
    SBRC    flags,mux_invert
    RJMP    mux_inverted2
mux_normal2:
.include "handle_parallel_pins_normal.inc"
    RJMP    wait_lpt_byte_main2
mux_inverted2:
.include "handle_parallel_pins_inverted.inc"
    RJMP    wait_lpt_byte_main2

wait_lpt_byte_main_trig2:
    reset_error_led
    store_parallel_data
    RET

;;; output_lpt_data ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine is used to return the least significant
; nibble of "sdata" to the ACK, PE, SEL and ERR pins
; of the parallel port.

output_lpt_data:
input_ack:
    SBRS    sdata,3
    RJMP    input_ack_false
input_ack_true:
    reset_ack
    RJMP    input_pe
input_ack_false:
    set_ack
input_pe:
    SBRS    sdata,2
    RJMP    input_pe_false
input_pe_true:
    set_pe
    RJMP    input_sel
input_pe_false:
    reset_pe
input_sel:
    SBRS    sdata,1
    RJMP    input_sel_false
input_sel_true:
    set_sel
    RJMP    input_err
input_sel_false:
    reset_sel
input_err:
    SBRS    sdata,0
    RJMP    input_err_false
input_err_true:
    reset_err
    RJMP    input_done
input_err_false:
    set_err
input_done:
    RET

;;; transmit_break ROUTINE ;;;;;;;;;;;;;;;;;;;;;;;;;;;;

; This routine will transmit a BREAK by taking TXD low
; and starting timer 0 which will run for the configured
; time (at least about 88 microsec = 704 cycles).
; The irq_timer0 routine will take TXD high again and
; will keep it high in order to transmit the MAB time
; for the configured time (at least 8 microsec = 64
; cycles). The configured amounts specify the number
; of microseconds 1 microsecond takes 8 cycles.
; The timer will then send the first output value
; (start code), so that the irq_transmit interrupt
; handler will be called.

transmit_break_mab:
; Initialize value of timer 0
    OUT     TCNT0,break_time
; For accurate timing, disable global interrupts for a short while
    CLI
; Temporarily disable transmitter
    CBI     UCR,TXEN
    reset_txd
; Start timer 1 running at 1/8th of clock frequency by setting CS01
    LDI     brk_temp,(0<<CS02)|(1<<CS01)|(0<<CS00) ; %1 cycle
    OUT     TCCR0,brk_temp          ; %1 cycle
; Reenable global interrupts
    SEI
    RET
